/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/
/*
    buffer.c

    This is for buffer management.

    2009-10-15	CF Lin
		new release
*/
#include "buffer.h"

void 
buffer_reset(BUFFER *b)
{
	if (b)
	{
		b->datasize=0;
		b->consumed=0;
	}
}

void
buffer_free (BUFFER * b)
{
    BUFFER *p;

    while (b)
    {
		p = b;
		b = b->next;
		if (p->data) FREE (p->data);
		FREE (p);
    }
}

BUFFER*
buffer_new (int size)
{
	BUFFER *b;

	b = MALLOC (sizeof (BUFFER));
	if (!b)	return NULL;

	b->data = MALLOC (size);
	if (!b->data)
	{
		FREE (b);
		b = NULL;
	}
	else
	{
		b->datasize = b->consumed = 0;
		b->datamax = size;
		b->next = NULL;
	}
	return b;
}

int
buffer_queue (BUFFER *b, void *src, int max, buffer_access_t bufx)
{
	int n;

	if (!b || !src || !bufx) 
		return 0;

	/* if there is un-consumed data, move it ahead */
	if (b->consumed)
	{
		int i;

		n = b->datasize - b->consumed;
		/* this should be fast */
		for (i=0; i < n; i++) b->data[i] = b->data[b->consumed++];
		b->consumed = 0;
		b->datasize = n;
	}
	n = b->datamax - b->datasize; /* available space */

	if (n)
	{
		if (max)
		{
			if (max < n) n = max;
			bufx(b->data+b->datasize, src, n); /* copy_from_user, Note : no return */
		}
		else
		{
			n = bufx(src, b->data+b->datasize, n); /* read_from_device */
		}
		b->datasize += n; /* move the pointer */
	}
	return n;
}

int
buffer_popup (BUFFER *b, void *dest, int max, buffer_access_t bufx)
{
	int n;

	if (!b || !dest || !bufx) 
		return 0;

	n = b->datasize - b->consumed; /* data */
	if (n)
	{
		if (max)
		{
			if (max < n) n = max;
			bufx(dest, b->data+b->consumed, n);  /* copy_to_user, Note : no return */
		}
		else
		{
			n = bufx(dest, b->data+b->consumed, n); /* write_to_device */
		}
		b->consumed += n; /* consumed bytes */
	}
	if (b->datasize==b->consumed)
	{
		b->datasize = b->consumed = 0;
	}
	return n;
}

BUFFER*
buffer_append (BUFFER *head, BUFFER *tail)
{
    BUFFER *b;

    if (!head)
	{
		b = tail;
	}
	else
	{
		b = head;
		while (head->next) head = head->next;
		head->next = tail;
	}
	return b;
}

BUFFER*
buffer_push (BUFFER *head, BUFFER *b)
{
	if (b)
	{
		b->next = head;
		head = b;
	}
	return head;
}
