#ifndef _CANBUS_DEF_H_
#define _CANBUS_DEF_H_
/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/
/*
    canbus_def.h

    This is a head file for canbus configuration.

    2009-10-15	CF Lin
		new release
*/
#include "driver.h"

/* 
 * Wade: we add spin_lock to avoid the 2 commands being interrupted !
 */
#ifdef LOCAL_BUS
#define reg_addr_write(port,offs,data) { \
	spin_lock(&port->lock_base_addr); \
	*(port->base_addr) = offs; \
	*(port->data_addr) = data; \
	spin_unlock(&port->lock_base_addr); }

#define reg_addr_read(port,offs,data) { \
	spin_lock(&port->lock_base_addr); \
	*(port->base_addr) = offs; \
	data = *(port->data_addr); \
	spin_unlock(&port->lock_base_addr); }

#elif defined PCI_BUS
/* 
 * Use ioread8 and iowrite8 to avoid porting problem 
 * only read global variable here, doesn't need spin_lock 
 */
#define reg_addr_write(port,offs,data) iowrite8(data, port->base_addr+offs)
#define reg_addr_read(port,offs,data) data = ioread8(port->base_addr+offs)
#endif 


#ifdef WIN32

/* timeout for read/write */
#define DEFAULT_TIMEOUT INFINITE

/*	We have a generic driver layer between the user program and the hardware chip. 
	This layer consists of a suite of functions XXX_.... 
	Redefine the names of these functions to fit the needs 
*/
#define	XXX_Init				CAN_Init
#define	XXX_Deinit				CAN_Deinit
#define	XXX_Open				CAN_Open
#define	XXX_Close				CAN_Close
#define	XXX_Read				CAN_Read
#define	XXX_Write				CAN_Write
#define	XXX_IOControl			CAN_IOControl

#define PHYSIC_BASE_MEMORY		(0x8A300000+0x20000000) /* base address */

#else
#if (! defined  CONFIG_MACH_MOXA_IA261) && (! defined X86)
#include	<asm/arch/ep93xx-regs.h>
#include	<asm/arch/gpio.h>
#endif	// CONFIG_MACH_MOXA_IA261

/* timeout for read/write */
#define DEFAULT_TIMEOUT 0

#define DEVICE_DRIVER_MAJOR		50
#define DEVICE_DRIVER_NAME		"can"

#define PHYSIC_BASE_MEMORY		0x20000000	/* base address */

#endif

#endif /* _CANBUS_DEF_H_ */
